searchData={"items":[{"type":"module","title":"Circuits.GPIO","doc":"Control GPIOs from Elixir\n\nSee the [Readme](README.md) for a tutorial and the [porting guide](PORTING.md)\nif updating from Circuits.GPIO v1.x.\n\nSimple example:\n\n```elixir\n# GPIO 2 is connected to GPIO 3\niex> {:ok, my_output_gpio} = Circuits.GPIO.open({\"gpiochip0\", 2}, :output)\niex> {:ok, my_input_gpio} = Circuits.GPIO.open({\"gpiochip0\", 3}, :input)\niex> Circuits.GPIO.write(my_output_gpio, 1)\n:ok\niex> Circuits.GPIO.read(my_input_gpio)\n1\niex> Circuits.GPIO.close(my_output_gpio)\niex> Circuits.GPIO.close(my_input_gpio)\n```","ref":"Circuits.GPIO.html"},{"type":"function","title":"Circuits.GPIO.backend_info/1","doc":"Return info about the low level GPIO interface\n\nThis may be helpful when debugging issues.","ref":"Circuits.GPIO.html#backend_info/1"},{"type":"function","title":"Circuits.GPIO.close/1","doc":"Release the resources associated with a GPIO\n\nThis is optional. The garbage collector will free GPIO resources that aren't\nin use, but this will free them sooner.","ref":"Circuits.GPIO.html#close/1"},{"type":"function","title":"Circuits.GPIO.enumerate/1","doc":"Return a list of accessible GPIOs\n\nEach GPIO is described in a `t:identifiers/0` map. Some fields in the map like\n`:location` and `:label` may be passed to `open/3` to use the GPIO. The map\nitself can also be passed to `open/3` and the function will figure out how to\naccess the GPIO.","ref":"Circuits.GPIO.html#enumerate/1"},{"type":"function","title":"Circuits.GPIO.gpio_spec?/1","doc":"Return if a term looks like a `gpio_spec`\n\nThis function only verifies that the term has the right shape to be a\n`t:gpio_spec/0`. Whether or not it refers to a usable GPIO is checked by\n`Circuits.GPIO.open/3`.","ref":"Circuits.GPIO.html#gpio_spec?/1"},{"type":"function","title":"Circuits.GPIO.identifiers/1","doc":"Return indentifying information about a GPIO\n\nSee `t:gpio_spec/0` for the ways of referring to GPIOs. If the GPIO is found,\nthis function returns information about the GPIO.","ref":"Circuits.GPIO.html#identifiers/1"},{"type":"macro","title":"Circuits.GPIO.is_gpio_spec/1","doc":"Guard version of `gpio_spec?/1`\n\nAdd `require Circuits.GPIO` to your source file to use this guard.","ref":"Circuits.GPIO.html#is_gpio_spec/1"},{"type":"function","title":"Circuits.GPIO.open/3","doc":"Open a GPIO\n\nSee `t:gpio_spec/0` for the ways of referring to GPIOs. Set `direction` to\neither `:input` or `:output`. If opening as an output, then be sure to set\nthe `:initial_value` option to minimize the time the GPIO is in the default\nstate.\n\nIf you're having trouble, see `enumerate/0` for available GPIOs. If you\nsuspect a hardware or driver issue, see `Circuits.GPIO.Diagnostics`.\n\nOptions:\n\n* :initial_value - Set to `0` or `1`. Only used for outputs. Defaults to `0`.\n* :pull_mode - Set to `:not_set`, `:pullup`, `:pulldown`, or `:none` for an\n   input pin. `:not_set` is the default.\n\nReturns `{:ok, handle}` on success.","ref":"Circuits.GPIO.html#open/3"},{"type":"function","title":"Circuits.GPIO.read/1","doc":"Read a GPIO's value\n\nThe value returned for GPIO's that are configured as outputs is undefined.\nBackends may choose not to support this.","ref":"Circuits.GPIO.html#read/1"},{"type":"function","title":"Circuits.GPIO.read_one/2","doc":"One line GPIO read\n\nThis is a convenience function that opens, reads, and closes a GPIO. It's\nintended to simplify one-off reads in code and for IEx prompt use.\n\nPrefer using handles in other situations.","ref":"Circuits.GPIO.html#read_one/2"},{"type":"function","title":"Circuits.GPIO.set_direction/2","doc":"Change the direction of the pin","ref":"Circuits.GPIO.html#set_direction/2"},{"type":"function","title":"Circuits.GPIO.set_interrupts/3","doc":"Enable or disable GPIO value change notifications\n\nNotifications are sent based on the trigger:\n\n* `:none` - No notifications are sent\n* `:rising` - Send a notification when the pin changes from 0 to 1\n* `:falling` - Send a notification when the pin changes from 1 to 0\n* `:both` - Send a notification on all changes\n\nAvailable Options:\n* `:suppress_glitches` - Not supported in Circuits.GPIO v2\n* `:receiver` - Process which should receive the notifications.\nDefaults to the calling process (`self()`)\n\nNotification messages look like:\n\n```\n{:circuits_gpio, gpio_spec, timestamp, value}\n```\n\nWhere `gpio_spec` is the `t:gpio_spec/0` passed to `open/3`, `timestamp` is an OS\nmonotonic timestamp in nanoseconds, and `value` is the new value.\n\nTimestamps are not necessarily the same as from `System.monotonic_time/0`.\nFor example, with the cdev backend, they're applied by the Linux kernel or\ncan be come from a hardware timer. Erlang's monotonic time is adjusted so\nit's not the same as OS monotonic time. The result is that these timestamps\ncan be compared with each other, but not with anything else.\n\nNOTE: You will need to store the `Circuits.GPIO` reference somewhere (like\nyour `GenServer`'s state) so that it doesn't get garbage collected. Event\nmessages stop when it gets collected. If you only get one message and you are\nexpecting more, this is likely the case.","ref":"Circuits.GPIO.html#set_interrupts/3"},{"type":"function","title":"Circuits.GPIO.set_pull_mode/2","doc":"Enable or disable an internal pull-up or pull-down resistor","ref":"Circuits.GPIO.html#set_pull_mode/2"},{"type":"function","title":"Circuits.GPIO.status/1","doc":"Return dynamic configuration and status information about a GPIO\n\nSee `t:gpio_spec/0` for the ways of referring to GPIOs. If the GPIO is found,\nthis function returns information about the GPIO.","ref":"Circuits.GPIO.html#status/1"},{"type":"function","title":"Circuits.GPIO.write/2","doc":"Set the value of a GPIO\n\nThe GPIO must be configured as an output.","ref":"Circuits.GPIO.html#write/2"},{"type":"function","title":"Circuits.GPIO.write_one/3","doc":"One line GPIO write\n\nThis is a convenience function that opens, writes, and closes a GPIO. It's\nintended to simplify one-off writes in code and for IEx prompt use.\n\nPrefer using handles in other situations.","ref":"Circuits.GPIO.html#write_one/3"},{"type":"type","title":"Circuits.GPIO.backend/0","doc":"Backends specify an implementation of a Circuits.GPIO.Backend behaviour\n\nThe second parameter of the Backend 2-tuple is a list of options. These are\npassed to the behaviour function call implementations.","ref":"Circuits.GPIO.html#t:backend/0"},{"type":"type","title":"Circuits.GPIO.controller/0","doc":"GPIO controller\n\nGPIO controllers manage one or more GPIO lines. They're referred to by\nstrings. For example, controllers are named `\"gpiochip0\"`, etc. for the\nLinux cdev backend. Other backends may have similar conventions or use\nthe empty string if there's only one controller.","ref":"Circuits.GPIO.html#t:controller/0"},{"type":"type","title":"Circuits.GPIO.direction/0","doc":"The GPIO direction (input or output)","ref":"Circuits.GPIO.html#t:direction/0"},{"type":"type","title":"Circuits.GPIO.gpio_spec/0","doc":"An identifier for a GPIO\n\nCall `Circuits.GPIO.enumerate/0` to see what GPIOs are available on your\ndevice. Several ways exist to refer to GPIOs due to variations in devices and\nprogrammer preference. Most Raspberry Pi models have labels like `\"GPIO26\"`.\nThe Raspberry Pi 5 has labels based on physical location (e.g., `\"PIN37\"` for\nGPIO 26.)\n\nOptions:\n\n1. `index` - Many examples exist where GPIOs are referred to by a GPIO\n   number. There are issues with this strategy since GPIO indices can change.\n   It is so common that it's still supported. Prefer other ways when you're\n   able to change code.\n2. `{controller_name, line_offset}` - Specify a line on a specific GPIO\n   controller. E.g., `{\"gpiochip0\", 10}`\n3. `label` - Specify a GPIO label. The first controller that has a\n   matching GPIO is used. This lets you move the mapping of GPIOs to\n   peripheral connections to a device tree file or other central place. E.g.,\n   `\"LED_ENABLE\"`\n4. `{controller_name, label}` - Specify both GPIO controller and GPIO labels.\n   E.g., `{\"gpiochip4\", \"PIO4\"}`","ref":"Circuits.GPIO.html#t:gpio_spec/0"},{"type":"type","title":"Circuits.GPIO.identifiers/0","doc":"Ways of referring to a GPIO\n\nIt's possible to refer to a GPIOs in many ways and this map contains\ninformation for doing that.  See `enumerate/1` and `identifiers/1` for\nquerying `Circuits.GPIO` for these maps.\n\nThe information in this map is backend specific. At a minimum, all backends\nprovide the `:location` field which is an unambiguous `t:gpio_spec/0` for use\nwith `open/3`.\n\nWhen provided, the `:label` field is a string name for the GPIO that should\nbe unique to the system but this isn't guaranteed. A common convention is to\nlabel GPIOs by their pin names in documentation or net names in schematics.\nThe Linux cdev backend uses labels from the device tree file.\n\nFields:\n\n* `:location` - this is the canonical gpio_spec for a GPIO.\n* `:label` - an optional label for the GPIO that may indicate what the GPIO is connected to\n  or be more helpful that the `:location`. It may be passed to `GPIO.open/3`.\n* `:controller` - the name or an alias for the GPIO controller. Empty string if unused","ref":"Circuits.GPIO.html#t:identifiers/0"},{"type":"type","title":"Circuits.GPIO.interrupt_options/0","doc":"Options for `set_interrupt/2`","ref":"Circuits.GPIO.html#t:interrupt_options/0"},{"type":"type","title":"Circuits.GPIO.label/0","doc":"A GPIO controller label or GPIO label\n\nLabels provide aliases for GPIO lines and controllers. They're\nsystem-specific. On Linux, labels are provided in device tree files.","ref":"Circuits.GPIO.html#t:label/0"},{"type":"type","title":"Circuits.GPIO.line_offset/0","doc":"GPIO line offset on a controller\n\nGPIOs are numbered based on how they're connected to a controller. The\ndetails are controller specific, but usually the first one is `0`, then `1`,\netc.","ref":"Circuits.GPIO.html#t:line_offset/0"},{"type":"type","title":"Circuits.GPIO.open_options/0","doc":"Options for `open/3`\n\n* `:initial_value` - the initial value of an output GPIO\n* `:pull_mode` - the initial pull mode for an input GPIO\n* `:force_enumeration` - Linux cdev-specific option to force a scan of\n  available GPIOs rather than using the cache. This is only for test purposes\n  since the GPIO cache should rescan as needed.","ref":"Circuits.GPIO.html#t:open_options/0"},{"type":"type","title":"Circuits.GPIO.pull_mode/0","doc":"Pull mode for platforms that support controllable pullups and pulldowns","ref":"Circuits.GPIO.html#t:pull_mode/0"},{"type":"type","title":"Circuits.GPIO.status/0","doc":"Dynamic GPIO configuration and status\n\nFields:\n\n* `:consumer` - if this GPIO is in use, this optional string gives a hint as to who is\n  using it.\n* `:direction` - whether this GPIO is an input or output\n* `:pull_mode` - if this GPIO is an input, then this is the pull mode","ref":"Circuits.GPIO.html#t:status/0"},{"type":"type","title":"Circuits.GPIO.trigger/0","doc":"Trigger edge for pin change notifications","ref":"Circuits.GPIO.html#t:trigger/0"},{"type":"type","title":"Circuits.GPIO.value/0","doc":"GPIO logic value (low = 0 or high = 1)","ref":"Circuits.GPIO.html#t:value/0"},{"type":"behaviour","title":"Circuits.GPIO.Backend","doc":"Backends provide the connection to the real or virtual GPIO controllers","ref":"Circuits.GPIO.Backend.html"},{"type":"callback","title":"Circuits.GPIO.Backend.backend_info/0","doc":"Return information about this backend","ref":"Circuits.GPIO.Backend.html#c:backend_info/0"},{"type":"callback","title":"Circuits.GPIO.Backend.enumerate/1","doc":"Return a list of GPIOs\n\nSee `t:GPIO.identifiers/0` for the information that is returned. The `options` contain\nbackend-specific options to help with enumeration.","ref":"Circuits.GPIO.Backend.html#c:enumerate/1"},{"type":"callback","title":"Circuits.GPIO.Backend.identifiers/2","doc":"Return identifying information about a GPIO\n\nSee `t:gpio_spec/0` for the ways of referring to GPIOs. The `options` contain\nbackend-specific options to help enumerating GPIOs.\n\nIf the GPIO is found, this function returns identifying information about the GPIO.","ref":"Circuits.GPIO.Backend.html#c:identifiers/2"},{"type":"callback","title":"Circuits.GPIO.Backend.open/3","doc":"Open a GPIO\n\nSee `t:gpio_spec/0` for the ways of referring to GPIOs. Set `direction` to\neither `:input` or `:output`. If opening as an output, then be sure to set\nthe `:initial_value` option to minimize the time the GPIO is in the default\nstate.\n\nOptions:\n\n* :initial_value - Set to `0` or `1`. Only used for outputs. Defaults to `0`.\n* :pull_mode - Set to `:not_set`, `:pullup`, `:pulldown`, or `:none` for an\n   input pin. `:not_set` is the default.\n\nReturns `{:ok, handle}` on success.","ref":"Circuits.GPIO.Backend.html#c:open/3"},{"type":"callback","title":"Circuits.GPIO.Backend.status/2","doc":"Return a GPIO's current status\n\nThis function returns how a GPIO is configured. The GPIO doesn't need to be\nopened. It's different from `gpio_identifiers/2` since it returns dynamic information\nwhereas `gpio_identifiers/2` only returns information about how to refer to a GPIO\nand where it exists in the system.\n\nSee `t:gpio_spec/0` for the ways of referring to GPIOs. The `options` contain\nbackend-specific options to help enumerating GPIOs.\n\nIf the GPIO is found, this function returns its status.","ref":"Circuits.GPIO.Backend.html#c:status/2"},{"type":"module","title":"Circuits.GPIO.CDev","doc":"Circuits.GPIO backend that uses the Linux CDev for controlling GPIOs\n\nThis is the default on Linux and Nerves. Nothing needs to be done to\nuse it on those platforms. If you need to be explicit, here's the\nconfiguration to force it:\n\n```elixir\nconfig :circuits_gpio, default_backend: Circuits.GPIO.CDev\n```\n\nIt takes one option, `:test`, that can be set to `true` to compile\nthe stub implementation that can be useful for testing.\n\n```elixir\nconfig :circuits_gpio, default_backend: {Circuits.GPIO.CDev, test: true}\n```","ref":"Circuits.GPIO.CDev.html"},{"type":"module","title":"Circuits.GPIO.Diagnostics","doc":"Runtime diagnostics\n\nThis module provides simple diagnostics to verify GPIO controller and\nimplementation differences. Along with the device that you're using, this is\nsuper helpful for diagnosing issues since some GPIO features aren't supposed\nto work on some devices.","ref":"Circuits.GPIO.Diagnostics.html"},{"type":"function","title":"Circuits.GPIO.Diagnostics.report/0","doc":"Reminder for how to use report/2","ref":"Circuits.GPIO.Diagnostics.html#report/0"},{"type":"function","title":"Circuits.GPIO.Diagnostics.report/2","doc":"Print a summary of the GPIO diagnostics\n\nConnect the pins referred to by `out_gpio_spec` and `in_gpio_spec` together.\nWhen using the cdev stub implementation, any pair of GPIOs can be used. For\nexample, run:\n\n```elixir\nCircuits.GPIO.Diagnostics.report({\"gpiochip0\", 0}, {\"gpiochip0\", 1})\n```\n\nThis function is intended for IEx prompt usage. See `run/2` for programmatic\nuse.","ref":"Circuits.GPIO.Diagnostics.html#report/2"},{"type":"function","title":"Circuits.GPIO.Diagnostics.run/2","doc":"Run GPIO tests and return a list of the results","ref":"Circuits.GPIO.Diagnostics.html#run/2"},{"type":"function","title":"Circuits.GPIO.Diagnostics.speed_test/1","doc":"Run GPIO API performance tests\n\nDisclaimer: There should be a better way than relying on the Circuits.GPIO\nwrite performance on nearly every device. Write performance shouldn't be\nterrible, though.","ref":"Circuits.GPIO.Diagnostics.html#speed_test/1"},{"type":"protocol","title":"Circuits.GPIO.Handle","doc":"Handle for referring to GPIOs\n\nUse the functions in `Circuits.GPIO` to read and write to GPIOs.","ref":"Circuits.GPIO.Handle.html"},{"type":"type","title":"Circuits.GPIO.Handle.t/0","doc":"All the types that implement this protocol.","ref":"Circuits.GPIO.Handle.html#t:t/0"},{"type":"module","title":"Circuits.GPIO.NilBackend","doc":"Circuits.GPIO backend when nothing else is available","ref":"Circuits.GPIO.NilBackend.html"},{"type":"extras","title":"Circuits - GPIO","doc":"# Circuits - GPIO\n\n[![Hex version](https://img.shields.io/hexpm/v/circuits_gpio.svg \"Hex version\")](https://hex.pm/packages/circuits_gpio)\n[![API docs](https://img.shields.io/hexpm/v/circuits_gpio.svg?label=hexdocs \"API docs\")](https://hexdocs.pm/circuits_gpio/Circuits.GPIO.html)\n[![CircleCI](https://circleci.com/gh/elixir-circuits/circuits_gpio.svg?style=svg)](https://circleci.com/gh/elixir-circuits/circuits_gpio)\n[![REUSE status](https://api.reuse.software/badge/github.com/elixir-circuits/circuits_gpio)](https://api.reuse.software/info/github.com/elixir-circuits/circuits_gpio)\n\n`Circuits.GPIO` lets you use GPIOs in Elixir.\n\n*This is the v2.0 development branch. If you're still using v1, please see the [maint-v1.x branch](https://github.com/elixir-circuits/circuits_gpio/tree/maint-v1.x).*\n\n`Circuits.GPIO` v2.0  is an almost backwards compatible update to `Circuits.GPIO`\nv1.x. Here's what's new:\n\n* Linux or Nerves are no longer required. In fact, the NIF supporting them won't\n  be compiled if you don't want it.\n* GPIOs can be enumerated to see what's available (See `Circuits.GPIO.enumerate/0`)\n* Linux and Nerves now use the Linux GPIO cdev subsystem rather than sysfs\n* GPIO pull mode setting for all platforms that support it rather than only Raspberry Pi\n* Develop using simulated GPIOs to work with LEDs and buttons with\n  [CircuitsSim](https://github.com/elixir-circuits/circuits_sim)\n\nIf you've used `Circuits.GPIO` v1.x, nearly all of your code will be the\nsame.`Circuits.GPIO` offers a substantial improvement by more descriptive GPIO\nspecs for identifying GPIOs. You can still refer to GPIOs by number. However,\nyou can also refer to GPIOs by labels and by which GPIO controller handles them.\nThe new `enumerate/0` can help with this.\n\nPlease review the [porting guide](PORTING.md) when upgrading from v1.x.","ref":"readme.html"},{"type":"extras","title":"Getting started on Nerves and Linux - Circuits - GPIO","doc":"If you're natively compiling `circuits_gpio` using Nerves or using a Linux-based\nSBC like a Raspberry Pi, everything should work like any other Elixir library.\nNormally, you would include `circuits_gpio` as a dependency in your `mix.exs`\nlike this:\n\n```elixir\ndef deps do\n  [{:circuits_gpio, \"~> 2.0\"}]\nend\n```\n\nOne common error on RaspberryPi OS is that the Erlang headers are missing\n(`erl_nif.h`), you may need to install erlang with `apt-get install erlang-dev`\nor build Erlang from source per instructions [here](http://elinux.org/Erlang).","ref":"readme.html#getting-started-on-nerves-and-linux"},{"type":"extras","title":"Examples - Circuits - GPIO","doc":"While `Circuits.GPIO` can support non-Nerves and non-Linux systems, the examples\nbelow were made using Nerves. Operation on other devices mostly differs on how\nto refer to GPIOs.\n\nThe following examples were tested on a Raspberry Pi that was connected to an\n[Erlang Embedded Demo Board](http://solderpad.com/omerk/erlhwdemo/). There's\nnothing special about either the demo board or the Raspberry Pi.","ref":"readme.html#examples"},{"type":"extras","title":"GPIO - Circuits - GPIO","doc":"A [General Purpose\nInput/Output](https://en.wikipedia.org/wiki/General-purpose_input/output) (GPIO)\nis just a wire that you can use as an input or an output. It can only be one of\ntwo values, 0 or 1. A 1 corresponds to a logic high voltage like 3.3 V and a 0\ncorresponds to 0 V. The actual voltage depends on the hardware.\n\nHere's an example of turning an LED on or off:\n\n![GPIO LED schematic](assets/images/schematic-gpio-led.png)\n\nTo turn on the LED that's connected to the net (or wire) labeled `GPIO18`, you\nneed to open it first. The first parameter to `Circuits.GPIO.open/2` is called a\nGPIO spec and identifies the GPIO. The Raspberry Pis are nice and provide string\nnames for GPIOs. Other boards are not as nice so you always have to check. The\nstring name for this GPIO is `\"GPIO18\"` (use `\"PIN12\"` on a Raspberry Pi 5).\n\n```elixir\niex> {:ok, gpio} = Circuits.GPIO.open(\"GPIO12\", :output)\n{:ok, %Circuits.GPIO.CDev{...}}\n\niex> Circuits.GPIO.write(gpio, 1)\n:ok\n\niex> Circuits.GPIO.close(gpio)\n:ok\n```\n\nThe call to `Circuits.GPIO.close/1` is not necessary, since the garbage\ncollector will free up unreferenced GPIOs. It's a good practice, though,\nsince backends can enforce exclusivity and prevent future opens from\nworking until the GC occurs.\n\nInput works similarly. Here's an example of a button with a pull down resistor\nconnected.\n\n![GPIO Button schematic](assets/images/schematic-gpio-button.png)\n\nIf you're not familiar with pull up or pull down resistors, they're resistors\nwhose purpose is to drive a wire high or low when the button isn't pressed. In\nthis case, it drives the wire low. Many processors have ways of configuring\ninternal resistors to accomplish the same effect without needing to add an\nexternal resistor. If you're using a Raspberry Pi, you can use [the built-in\npull-up/pull-down resistors](#internal-pull-uppull-down).\n\nThe code looks like this in `Circuits.GPIO`:\n\n```elixir\niex> {:ok, gpio} = Circuits.GPIO.open(\"GPIO17\", :input)\n{:ok, %Circuits.GPIO.CDev{...}}\n\niex> Circuits.GPIO.read(gpio)\n0\n\n# Push the button down\n\niex> Circuits.GPIO.read(gpio)\n1\n```\n\nIf you'd like to get a message when the button is pressed or released, call the\n`set_interrupts` function. You can trigger on the `:rising` edge, `:falling`\nedge or `:both`.\n\n```elixir\niex> Circuits.GPIO.set_interrupts(gpio, :both)\n:ok\n\niex> flush\n{:circuits_gpio, \"GPIO17\", 1233456, 1}\n{:circuits_gpio, \"GPIO17\", 1234567, 0}\n:ok\n```\n\nNote that after calling `set_interrupts`, the calling process will receive an\ninitial message with the state of the pin. This prevents the race condition\nbetween getting the initial state of the pin and turning on interrupts. Without\nit, you could get the state of the pin, it could change states, and then you\ncould start waiting on it for interrupts. If that happened, you would be out of\nsync.\n\n#","ref":"readme.html#gpio"},{"type":"extras","title":"Internal pull-up/pull-down - Circuits - GPIO","doc":"To connect or disconnect an internal [pull-up or pull-down resistor](https://github.com/raspberrypilearning/physical-computing-guide/blob/master/pull_up_down.md) to a GPIO\npin, call the `set_pull_mode` function.\n\n```elixir\niex> Circuits.GPIO.set_pull_mode(gpio, pull_mode)\n:ok\n```\n\nValid `pull_mode` values are `:none` `:pullup`, or `:pulldown`\n\nNote that `set_pull_mode` is platform dependent, and currently only works for\nRaspberry Pi hardware.  Calls to `set_pull_mode` on other platforms will have no\neffect.  The internal pull-up resistor value is between 50K and 65K, and the\npull-down is between 50K and 60K.  It is not possible to read back the current\nPull-up/down settings, and GPIO pull-up pull-down resistor connections are\nmaintained, even when the CPU is powered down.","ref":"readme.html#internal-pull-up-pull-down"},{"type":"extras","title":"GPIO Specs - Circuits - GPIO","doc":"`Circuits.GPIO` v2.0 supports a new form of specifying how to open a GPIO called\na `t:gpio_spec/0`. These specs are very flexible and allow for GPIOs to be\nopened by number, a string label, or a tuple that includes both the GPIO\ncontroller hardware name and a line offset.\n\nThe contents of a `gpio_spec` depend on the backend. When running on Nerves or\na Linux machine, `Circuits.GPIO` uses the Linux gpio-cdev backend. This backend\nprefers the use of GPIO controller/line offset tuples and labels. For backwards\ncompatibility, it somewhat supports use of the *older* pin numbering scheme.\n\nThe GPIO controller part of the tuple is usually some variation on `\"gpiochip0\"`\nthat depends on what controllers are available under `/dev`. The line offset is\na the line offset of the GPIO on that controller.\n\n```elixir\niex> {:ok, gpio} = Circuits.GPIO.open({\"gpiochip0\", 1}, :input)\n{:ok, %Circuits.GPIO.CDev{...}}\n```\n\nWhen the Linux device tree is configured with GPIO labels, you can use those instead:\n\n```elixir\niex> {:ok, gpio} = Circuits.GPIO.open(\"special-name-for-pin-1\")\n{:ok, %Circuits.GPIO.CDev{...}}\n```\n\nIf you're deploying to multiple types of devices and you can set labels in the\ndevice tree, labels make it really easy for code using `Circuits.GPIO` to just\nuse the right GPIO.\n\nLabels are not guaranteed to be unique, so if your device defines one twice,\n`Circuits.GPIO` will use the first GPIO it finds that has the specified label.\n\nSee [Enumeration](#enumeration) for listing out all available `gpio_specs` for\nyour device.","ref":"readme.html#gpio-specs"},{"type":"extras","title":"Enumeration - Circuits - GPIO","doc":"`Circuits.GPIO` v2.0 supports a new function, `enumerate/0`, which lists every\nknown GPIO pin.\n\nFor Nerves and Linux users, the `gpio-cdev` subsystem maintains the official\nlist. See the [Official DeviceTree documentation for\nGPIOs](https://elixir.bootlin.com/linux/v6.6.6/source/Documentation/devicetree/bindings/gpio/gpio.txt)\nfor more information on how to configure the fields of this struct for your own\nsystem.\n\nHere's an example:\n\n```elixir\niex> Circuits.GPIO.enumerate()\n[\n  %{\n    location: {\"gpiochip0\", 0},\n    label: \"ID_SDA\",\n    controller: \"pinctrl-bcm2835\"\n  },\n  %{\n    location: {\"gpiochip0\", 1},\n    label: \"ID_SCL\",\n    controller: \"pinctrl-bcm2835\"\n  },\n  %{\n    location: {\"gpiochip0\", 2},\n    label: \"SDA1\",\n    controller: \"pinctrl-bcm2835\"\n  },\n  ...\n]\n```\n\nThe `:location` can always be passed as the first parameter to\n`Circuits.GPIO.open/3`. You may find the `:label` field more descriptive to use,\nthough.","ref":"readme.html#enumeration"},{"type":"extras","title":"Convenience functions - Circuits - GPIO","doc":"Having to `open` *then* `read` and `write` can be a cumbersome for one-off GPIO\naccess in code and when working at the IEx prompt. Circuits v2.0 has a pair of\nnew functions to help:\n\n```elixir\niex> alias Circuits.GPIO\niex> GPIO.write_one(\"special-name-for-pin-1\", 1)\n:ok\niex> GPIO.read_one(\"special-name-for-pin-2\")\n1\n```\n\nThese functions get passed a `t:gpio_spec/0` just like `open/3` and internally\nopen the GPIO and read or write it. Importantly, they `close` the GPIO when done\nto avoid reserving the GPIO any longer than necessary.\n\nPlease note that this is not a performant way of reading or writing the same\nGPIO more than once. Opening a GPIO takes much longer than reading or writing an\nalready opened one, so if these are used in tight loops, the open overhead will\ndominate (>99% of the time taken in a trivial benchmark.)","ref":"readme.html#convenience-functions"},{"type":"extras","title":"Testing - Circuits - GPIO","doc":"`Circuits.GPIO` supports a \"stub\" hardware abstraction layer on platforms\nwithout GPIO support and when `MIX_ENV=test`. The stub allows for some limited\nunit testing without real hardware.\n\nTo use it, first check that you're using the \"stub\" HAL:\n\n```elixir\niex> Circuits.GPIO.backend_info()\n%{name: :stub, pins_open: 0}\n```\n\nThe stub HAL has 64 GPIOs. Each pair of GPIOs is connected. For example, GPIO 0\nis connected to GPIO 1. If you open GPIO 0 as an output and GPIO 1 as an input,\nyou can write to GPIO 0 and see the result on GPIO 1. Here's an example:\n\n```elixir\niex> {:ok, gpio0} = Circuits.GPIO.open({\"gpiochip0\", 0}, :output)\n{:ok, %Circuits.GPIO.CDev{...}}\niex> {:ok, gpio1} = Circuits.GPIO.open({\"gpiochip0\", 1}, :input)\n{:ok, %Circuits.GPIO.CDev{...}}\niex> Circuits.GPIO.read(gpio1)\n0\niex> Circuits.GPIO.write(gpio0, 1)\n:ok\niex> Circuits.GPIO.read(gpio1)\n1\n```\n\nThe stub HAL is fairly limited, but it does support interrupts.\n\nIf `Circuits.GPIO` is used as a dependency the stub may not be present. To\nmanually enable it, set `CIRCUITS_MIX_ENV` to `test` and rebuild\n`circuits_gpio`.","ref":"readme.html#testing"},{"type":"extras","title":"FAQ - Circuits - GPIO","doc":"#","ref":"readme.html#faq"},{"type":"extras","title":"Where can I get help? - Circuits - GPIO","doc":"Most issues people have are on how to communicate with hardware for the first\ntime. Since `Circuits.GPIO` is a thin wrapper on the Linux CDev GPIO interface,\nyou may find help by searching for similar issues when using Python or C.\n\nFor help specifically with `Circuits.GPIO`, you may also find help on the nerves\nchannel on the [elixir-lang Slack](https://elixir-lang.slack.com/).  Many\n[Nerves](http://nerves-project.org) users also use `Circuits.GPIO`.\n\n#","ref":"readme.html#where-can-i-get-help"},{"type":"extras","title":"I tried turning on and off a GPIO as fast as I could. Why was it slow? - Circuits - GPIO","doc":"Please don't do that - there are so many better ways of accomplishing whatever\nyou're trying to do:\n\n1. If you're trying to drive a servo or dim an LED, look into PWM. Many\n   platforms have PWM hardware and you won't tax your CPU at all. If your\n   platform is missing a PWM, several chips are available that take I2C commands\n   to drive a PWM output.\n2. If you need to implement a wire level protocol to talk to a device, look for\n   a Linux kernel driver. It may just be a matter of loading the right kernel\n   module.\n3. If you want a blinking LED to indicate status, `gpio` really should\n   be fast enough to do that, but check out Linux's LED class interface. Linux\n   can flash LEDs, trigger off events and more. See [nerves_leds](https://github.com/nerves-project/nerves_leds).\n\nIf you're still intent on optimizing GPIO access, you may be interested in\n[gpio_twiddler](https://github.com/fhunleth/gpio_twiddler). Note that the\ntwiddler doc is dated. Circuits.GPIO v2 should be faster than v1.\n\n#","ref":"readme.html#i-tried-turning-on-and-off-a-gpio-as-fast-as-i-could-why-was-it-slow"},{"type":"extras","title":"Can I develop code that uses GPIO on my laptop? - Circuits - GPIO","doc":"The intended way to support this is to have a custom `Circuits.GPIO.Backend`\nthat runs on your laptop. The\n[CircuitsSim](https://github.com/elixir-circuits/circuits_sim) is an example of\na project that provides simulated LEDs and buttons.","ref":"readme.html#can-i-develop-code-that-uses-gpio-on-my-laptop"},{"type":"extras","title":"License - Circuits - GPIO","doc":"All original source code in this project is licensed under Apache-2.0.\n\nAdditionally, this project follows the [REUSE recommendations](https://reuse.software)\nand labels so that licensing and copyright are clear at the file level.\n\nExceptions to Apache-2.0 licensing are:\n\n* Configuration and data files are licensed under CC0-1.0\n* Documentation files are CC-BY-4.0\n* Erlang Embedded board images are Solderpad Hardware License v0.51.","ref":"readme.html#license"},{"type":"extras","title":"Porting","doc":"# Porting","ref":"porting.html"},{"type":"extras","title":"Upgrading Circuits.GPIO 1.0 projects to 2.0 - Porting","doc":"Circuits.GPIO 2.0 supports alternative GPIO hardware and the ability to mock or\nemulate devices via backends. The Linux cdev backend is the default and usage is\nsimilar to Circuits.GPIO 1.0. Most projects won't need any changes other than to\nupdate the dependency in `mix.exs`. If upgrading a library, The following\ndependency specification is recommended to allow both `circuits_gpio` versions:\n\n```elixir\n   {:circuits_gpio, \"~> 2.0 or ~> 1.0\"}\n```\n\nThe following breaking changes were made:\n\n1. `Circuits.GPIO.open/3` opens an exclusive reference to the GPIO. It's no\n   longer possible to have multiple processes open the same GPIO. This sometimes\n   comes up when GPIOs are not closed when they're done being used. The garbage\n   collector will eventually close the GPIO, so you may see intermittent failed\n   opens. The solution is either to open once and pass the handle around or to\n   explicitly close handles after usage.\n2. `Circuits.GPIO.open/3` accepts more general pin specifications called\n   `gpio_spec`s. This allows you to specify GPIO controllers and refer to pins\n   by labels. Please see `t:Circuits.GPIO.gpio_spec/0` since referring to pins\n   by number is brittle and has broken in the past.\n3. `Circuits.GPIO.open/3` no longer can preserve the previous output value on\n   a pin automatically. I.e., `initial_value: :not_set` is no longer supported.\n4. Reading the values of output GPIOs is not supported. This does have a chance\n   of working depending on the backend, but it's no longer a requirement of the\n   API since some backends may not be readable. The workaround is to cache what\n   you wrote.\n5. `Circuits.GPIO.set_interrupts/3` does not send an initial notification.\n   Notifications are ONLY sent on GPIO transitions now.\n6. The `stub` implementation still exists and is useful for testing the cdev NIF\n   interface. It's possible to have alternative GPIO backends now. If you have\n   simple needs, the `stub` is convenient since it provides pairs of connected\n   GPIOs (e.g., 0 and 1, 2 and 3, etc.).\n7. `Circuits.GPIO.pin/1` is no longer available.\n\nYou should hopefully find that the semantics of API are more explicit now or at\nleast the function documentation is more clear. This was necessary to support\nmore backends without requiring backend authors to implement features that are\ntrickier than you'd expect.\n\nIf you find that you have to make any other changes, please let us know via an\nissue or PR so that other users can benefit.","ref":"porting.html#upgrading-circuits-gpio-1-0-projects-to-2-0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v2.0.1 - 2024-01-13 - Changelog","doc":"* Bug fixes\n  * Fix race condition when loading NIF. If two processes caused the NIF to be\n    loaded at the same time, then it was possible for one to return an error.\n  * Remove tracking of the number of open pins from the cdev backend to not need\n    to synchronize access to the count. This feature really was only used for\n    the unit tests.","ref":"changelog.html#v2-0-1-2024-01-13"},{"type":"extras","title":"v2.0.0 - 2024-01-11 - Changelog","doc":"This is a major update to Circuits.GPIO that modernizes the API, restricts usage\nto Nerves and Linux, and updates the Linux/Nerves backend to the Linux GPIO cdev\ninterface.\n\nIt is mostly backwards compatible with Circuits.GPIO v1. Please see `PORTING.md`\nfor upgrade instructions.\n\n* New features\n  * Support alternative backends for different operating systems or for\n    simulated hardware. The Linux cdev backend can be compiled out.\n\n  * `Circuits.GPIO.open/3` is much more flexible in how GPIOs are identified.\n    Specifying GPIOs by number still works, but it's now possible to specify\n    GPIOs by string labels and by tuples that contain the GPIO controller name\n    and index. See `t:gpio_spec/0` and the `README.md` for details.\n\n  * List out available GPIOs with `Circuits.GPIO.enumerate/0`. Other helper\n    functions are available for getting more information about each GPIO too.\n\n  * Specify pull modes in general rather than only Raspberry Pis on Linux and\n    Nerves\n\n  * Easily do one-off reads and writes with `Circuits.GPIO.read_one/2` and\n    `Circuits.GPIO.write_one/3`\n\n  * Improved performance on Nerves and Linux; kernel-applied timestamping of\n    GPIO input events\n\n  * Add `Circuits.GPIO.Diagnostics` to automate runtime testing\n\n* Changes\n  * More consistent error returns. Unexpected errors return `{:errno, value}`\n    tuples to help correlate errors to low level docs\n  * Deferred loading of the NIF to simplify debugging of GPIO backends.\n    Segfaults crash on first use of `Circuits.GPIO` rather than on load.","ref":"changelog.html#v2-0-0-2024-01-11"},{"type":"extras","title":"v1.1.0 - 2022-12-31 - Changelog","doc":"* Changes\n  * Remove Erlang convenience functions since no one used them\n  * Require Elixir 1.10 or later. Previous versions probably work, but won't be\n    supported. This opens up the possibility of using Elixir 1.10+ features in\n    future releases.","ref":"changelog.html#v1-1-0-2022-12-31"},{"type":"extras","title":"v1.0.1 - 2022-7-26 - Changelog","doc":"* Bug fixes\n  * On at least one device, the pin direction reported by Linux does not match\n    the actual pin direction. This release forces the pin direction for inputs.\n    It cannot do this for outputs, since setting a pin to output has a side\n    effect on the pin state. This technically is a bug in a Linux driver port,\n    but it was harmless to workaround for inputs so that's what's done. Thanks\n    to @pojiro for investigating and fixing this issue.","ref":"changelog.html#v1-0-1-2022-7-26"},{"type":"extras","title":"v1.0.0 - 2021-10-20 - Changelog","doc":"This release only changes the version number. No code has changed.","ref":"changelog.html#v1-0-0-2021-10-20"},{"type":"extras","title":"v0.4.8 - Changelog","doc":"This release only has doc and build output cleanup. No code has changed.","ref":"changelog.html#v0-4-8"},{"type":"extras","title":"v0.4.7 - Changelog","doc":"* Bug fixes\n  * Fix hang when unloading the NIF. This bug caused `:init.stop` to never\n    return and anything else that would try to unload this module.\n  * Fix C compiler warnings with OTP 24\n\nThe minimum Elixir version has been changed from 1.4 to 1.6. Elixir 1.4 might\nstill work, but it's no longer being verified on CI.","ref":"changelog.html#v0-4-7"},{"type":"extras","title":"v0.4.6 - Changelog","doc":"* Bug fixes\n  * Fix quoting issue that was causing failures on Yocto. Thanks to Zander\n    Erasmus for this.","ref":"changelog.html#v0-4-6"},{"type":"extras","title":"v0.4.5 - Changelog","doc":"* Bug fixes\n  * Opening a GPIO to read its state won't clear the interrupt status of another\n    listener. Registering for interrupts a second time will still clear out the\n    first registree, though. That's a limitation of the interface. However, for\n    debugging, it can be nice to look at a GPIO without affecting the program\n    and this change allows for that.","ref":"changelog.html#v0-4-5"},{"type":"extras","title":"v0.4.4 - Changelog","doc":"* Bug fixes\n  * Add -fPIC to compilation flags to fix build with `nerves_system_x86_64` and\n    other environments using the Musl C toolchains","ref":"changelog.html#v0-4-4"},{"type":"extras","title":"v0.4.3 - Changelog","doc":"* Bug fixes\n  * Fix GPIO glitch suppression when interrupts were enabled. Glitch suppression\n    filters out transitions on a GPIO line that are too fast for Linux and the\n    NIF to see both the rising and falling edges. Turning it off synthesizes\n    events. You can identify synthesized events since they have the same\n    timestamp to the nanosecond. See `Circuits.GPIO.set_interrupts/3`.\n\n* Improvement\n  * It's possible to enable the \"stub\" on Linux by setting\n    `CIRCUITS_MIX_ENV=test`. This can be useful for unit testing code that uses\n    Circuits.GPIO. Thanks to Enrico Rivarola for adding this!","ref":"changelog.html#v0-4-3"},{"type":"extras","title":"v0.4.2 - Changelog","doc":"* Bug fixes\n  * Fix pullup/pulldown support for the Raspberry Pi 4","ref":"changelog.html#v0-4-2"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"* Bug fixes\n  * Fix a race condition on Raspbian where Circuits.GPIO would try to open the\n    GPIO sysfs file before udev had a chance to fix its permissions.\n  * Fix RPi platform detection on Raspbian so that pull-ups/pull-downs work\n    without passing any flags.","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"The GPIO interrupt notification messages have been changed for consistency with\nother circuits projects. The initial element of the tuple is now\n`:circuits_gpio`, so messages will look like:\n\n`{:circuits_gpio, 19, 83268239, 1}`\n\nPlease update your project if you call `set_interrupts/2`.\n\nNo more backwards incompatible changes are expected until after 1.0.","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"* Bug fixes\n  * Build C source under the `_build` directory so that changing targets\n    properly rebuilds the C code as well as the Elixir.","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"* New features\n  * Support `pull_mode` initializion in `open/3`.","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"* New features\n  * Add support for opening GPIOs to an initial value or to not change the\n    value. This removes a glitch if you want the GPIO to start out high (the\n    default was low) or if you want the GPIO to keep its current value.","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"Initial release to hex.","ref":"changelog.html#v0-1-0"}],"content_type":"text/markdown"}